@@ -11,12 +11,13 @@
 #' genes and exons in the locus.
 #' 
 #' @param gene Optional character value specifying which gene to view. Either
-#'   `gene` or `xrange` plus `seqname` must be specified.
-#' @param data Dataset (data.frame or data.table) to use for plot.
+#'   `gene`, or `xrange` plus `seqname`, or `index_snp` must be specified.
+#' @param data Dataset (data.frame or data.table) to use for plot. If
+#'   unspecified or `NULL`, gene track information alone is returned.
 #' @param xrange Optional vector of genomic position range for the x axis.
 #' @param seqname Optional, specifies which chromosome to plot.
 #' @param flank Single value or vector with 2 values for how much flanking 
-#' region left and right of the gene to show. Defaults to 50,000.
+#' region left and right of the gene to show. Defaults to 100kb.
 #' @param fix_window Optional alternative to `flank`, which allows users to
 #'   specify a fixed genomic window centred on the specified gene. Both `flank`
 #'   and `fix_window` cannot be specified simultaneously.
@@ -38,7 +39,8 @@
 #' @param labs Determines which column in `data` contains SNP rs IDs.
 #' If `NULL` tries to autodetect the column.
 #' @param index_snp Specifies the index SNP. If not specified, the SNP with the
-#'   lowest P value is selected.
+#'   lowest P value is selected. Can be used to specify locus region instead of
+#'   specifying `gene`, or `seqname` and `xrange`.
 #' @param LD Optional character value to specify which column in `data` contains
 #'   LD information.
 #' @return Returns an object of class 'locus' ready for plotting, containing 
@@ -48,7 +50,8 @@
 #' column names for chromosome, position, SNP ID, p-value or variable for
 #' plotting on y axis,
 #' locus gene information from Ensembl and
-#' locus exon information from Ensembl.
+#' locus exon information from Ensembl. If `data` is `NULL` then gene track
+#' information alone is returned.
 #' @examples
 #' ## Bioconductor package EnsDb.Hsapiens.v75 is needed for these examples
 #' if(require(EnsDb.Hsapiens.v75)) {
@@ -64,13 +67,14 @@
 #' @importFrom ensembldb genes exons
 #' @importFrom BiocGenerics start end
 #' @importFrom AnnotationFilter GeneNameFilter AnnotationFilterList 
-#' SeqNameFilter GeneIdFilter
+#' SeqNameFilter GeneIdFilter TxStartFilter TxEndFilter ExonStartFilter 
+#' ExonEndFilter
 #' @importFrom GenomeInfoDb seqlengths
 #' @importFrom memoise memoise
 #' @export
 
 locus <- function(gene = NULL,
-                  data,
+                  data = NULL,
                   xrange = NULL, seqname = NULL,
                   flank = NULL, fix_window = NULL,
                   ens_db,
@@ -87,91 +91,139 @@ locus <- function(gene = NULL,
   } else edb <- ens_db
   if (!is.null(flank) & !is.null(fix_window))
     stop("both `flank` and `fix_window` cannot be specified at the same time")
-  if (is.null(flank)) flank <- 5e4
+  if (is.null(flank)) flank <- 1e5
   flank <- rep_len(flank, 2)
+  
   if (!is.null(gene)) {
-    locus <- genes(edb, filter = GeneNameFilter(gene))
+    locus <- genes(edb, filter = AnnotationFilterList(
+      GeneNameFilter(gene),
+      SeqNameFilter(c(1:22, 'X', 'Y'))
+    ))
+    
+    if(length(locus) > 1) {
+      warning(sprintf('Identified %d genes matching name \'%s\', taking first\n',
+                      length(locus), gene))
+      locus <- locus[1]
+    }
     seqname <- names(seqlengths(locus))
     if (is.null(fix_window)) {
       xrange <- c(start(locus) - flank[1], end(locus) + flank[2])
     } else {
       m <- mean(c(start(locus), end(locus)))
-      xrange <- c(m - fix_window/2, m + fix_window/2)
+      xrange <- as.integer(c(m - fix_window/2, m + fix_window/2))
+    }
+    xrange[xrange < 0] <- 0
+  }
+  
+  if (!is.null(data)) {
+    # autodetect headings
+    if (is.null(chrom)) {
+      w <- grep("chr", colnames(data), ignore.case = TRUE)
+      if (length(w) == 1) {
+        chrom <- colnames(data)[w]
+      } else stop("unable to autodetect chromosome column")
+    }
+    if (is.null(pos)) {
+      w <- grep("pos", colnames(data), ignore.case = TRUE)
+      if (length(w) == 1) {
+        pos <- colnames(data)[w]
+      } else stop("unable to autodetect SNP position column")
+    }
+    if (!is.null(p) && !is.null(yvar)) stop("cannot specify both `p` and `yvar`")
+    if (is.null(p) && is.null(yvar)) {
+      if ("p" %in% colnames(data)) {
+        p <- "p"
+      } else {
+        w <- grep("^p?val", colnames(data), ignore.case = TRUE)
+        if (length(w) == 1) {
+          p <- colnames(data)[w]
+        } else stop("unable to autodetect p-value column")
+      }
+    }
+    if (is.null(labs)) {
+      w <- grep("rs?id|SNP", colnames(data), ignore.case = TRUE)
+      if (length(w) == 1) {
+        labs <- colnames(data)[w]
+      } else stop("unable to autodetect SNP id column")
+    }
+    
+    # check headings
+    if (!chrom %in% colnames(data)) {
+      stop("Column specified by `chrom` not found in `data`")}
+    if (!pos %in% colnames(data)) {
+      stop("Column specified by `pos` not found in `data`")}
+    if (is.null(yvar) && !p %in% colnames(data)) {
+      stop("Column specified by `p` not found in `data`")}
+    if (!labs %in% colnames(data)) {
+      stop("Column specified by `labs` not found in `data`")}
+    if (!is.null(yvar)) {
+      if (!yvar %in% colnames(data)) {
+        stop("Column specified by `yvar` not found in `data`")
+      }
+    }
+    
+    if (!is.null(index_snp) & is.null(gene) & is.null(seqname) & is.null(xrange)) {
+      # region based on index SNP
+      if (!index_snp %in% data[, labs])
+        stop("SNP specified by `index_snp` not found")
+      ind <- which(data[, labs] == index_snp)
+      if (length(ind) > 1) message("SNP found more than once")
+      seqname <- data[ind[1], chrom]
+      snp_pos <- data[ind[1], pos]
+      xrange <- if (is.null(fix_window)) {
+        c(snp_pos - flank[1], snp_pos + flank[2])
+      } else {
+        as.integer(c(snp_pos - fix_window/2, snp_pos + fix_window/2))
+      }
+      xrange[xrange < 0] <- 0
     }
   }
-  xrange <- as.integer(xrange)
+  
   if (is.null(xrange) | is.null(seqname)) stop('No locus specified')
   message("Chromosome ", seqname, ", position ", xrange[1], " to ", xrange[2])
   
-  # autodetect headings
-  if (is.null(chrom)) {
-    w <- grep("chr", colnames(data), ignore.case = TRUE)
-    if (length(w) == 1) {
-      chrom <- colnames(data)[w]
-    } else stop("unable to autodetect chromosome column")
-  }
-  if (is.null(pos)) {
-    w <- grep("pos", colnames(data), ignore.case = TRUE)
-    if (length(w) == 1) {
-      pos <- colnames(data)[w]
-    } else stop("unable to autodetect SNP position column")
-  }
-  if (!is.null(p) && !is.null(yvar)) stop("cannot specify both `p` and `yvar`")
-  if (is.null(p) && is.null(yvar)) {
-    if ("p" %in% colnames(data)) {
-      p <- "p"
-    } else {
-      w <- grep("^p?val", colnames(data), ignore.case = TRUE)
-      if (length(w) == 1) {
-        p <- colnames(data)[w]
-      } else stop("unable to autodetect p-value column")
+  if (!is.null(data)) { 
+    data <- data[data[, chrom] == seqname &
+                   data[, pos] > xrange[1] & data[, pos] < xrange[2], ]
+    if (is.null(yvar)) {
+      data$logP <- -log10(data[, p])
+      yvar <- "logP"
     }
-  }
-  if (is.null(labs)) {
-    w <- grep("rs?id|SNP", colnames(data), ignore.case = TRUE)
-    if (length(w) == 1) {
-      labs <- colnames(data)[w]
-    } else stop("unable to autodetect SNP id column")
-  }
-  
-  # check headings
-  if (!chrom %in% colnames(data)) {
-    stop("Column specified by `chrom` not found in `data`")}
-  if (!pos %in% colnames(data)) {
-    stop("Column specified by `pos` not found in `data`")}
-  if (is.null(yvar) && !p %in% colnames(data)) {
-    stop("Column specified by `p` not found in `data`")}
-  if (!labs %in% colnames(data)) {
-    stop("Column specified by `labs` not found in `data`")}
-  if (!is.null(yvar)) {
-    if (!yvar %in% colnames(data)) {
-      stop("Column specified by `yvar` not found in `data`")
+    data <- as.data.frame(data)
+
+    if (nrow(data) == 0) stop("Locus contains no SNPs/datapoints")
+
+    if (is.null(index_snp)) index_snp <- data[which.max(data[, yvar]), labs]
+    if (is.character(LD)) {
+      colnames(data)[which(colnames(data) == LD)] <- "ld"
     }
+    message(nrow(data), " SNPs/datapoints")
   }
   
-  data <- data[data[, chrom] == seqname &
-                 data[, pos] > xrange[1] & data[, pos] < xrange[2], ]
-  if (is.null(yvar)) {
-    data$logP <- -log10(data[, p])
-    yvar <- "logP"
-  }
-  data <- as.data.frame(data)
-  if (is.null(index_snp)) index_snp <- data[which.max(data[, yvar]), labs]
-  if (is.character(LD)) {
-    colnames(data)[which(colnames(data) == LD)] <- "ld"
-  }
-  message(nrow(data), " SNPs/datapoints")
-  
+  seqname <- gsub("chr|[[:punct:]]", "", seqname, ignore.case = TRUE)
+  if (!seqname %in% c(1:22, "X", "Y")) 
+    warning("`seqname` refers to a non-conventional chromosome")
   TX <- ensembldb::genes(edb, filter = AnnotationFilterList(
-    SeqNameFilter(c(seq_len(22), "X", "Y")),
+    SeqNameFilter(seqname),
+    TxStartFilter(xrange[2], condition = "<"),
+    TxEndFilter(xrange[1], condition = ">"),
     GeneIdFilter("ENSG", "startsWith")))
   TX <- data.frame(TX)
   TX <- TX[! is.na(TX$start), ]
-  TX <- TX[TX$seqnames == seqname, ]
-  TX <- TX[TX$end > xrange[1], ]
-  TX <- TX[TX$start < xrange[2], ]
-  EX <- ensembldb::exons(edb, filter = GeneIdFilter(TX$gene_id))
+  TX <- TX[!duplicated(TX$gene_id), ]
   
+  if (nrow(TX) == 0) {
+    message("No gene transcripts")
+    # Creating empty exons object here in suitable format
+    EX <- ensembldb::exons(edb, filter = AnnotationFilterList(
+      SeqNameFilter(seqname),
+      ExonStartFilter(xrange[2], condition = "<"),
+      ExonEndFilter(xrange[1], condition = ">"),
+      GeneIdFilter("ENSG", "startsWith")))
+  } else {
+    EX <- ensembldb::exons(edb, filter = GeneIdFilter(TX$gene_id))
+  }
+
   loc <- list(seqname = seqname, xrange = xrange, gene = gene,
               ens_db = ens_db,
               chrom = chrom, pos = pos, p = p, yvar = yvar, labs = labs,
@@ -184,13 +236,21 @@ locus <- function(gene = NULL,
 
 #' @export
 summary.locus <- function(object, ...) {
-  cat("Gene", object$gene, "\n")
+  if (!is.null(object$gene)) {
+    cat("Gene", object$gene, "\n")
+  } else if (!is.null(object$index_snp)) {
+    cat("Index SNP", object$index_snp, "\n")
+  }
   cat(paste0("Chromosome ", object$seqname, ", position ",
              format(object$xrange[1], big.mark=","), " to ",
              format(object$xrange[2], big.mark=","), "\n"))
-  cat(nrow(object$data), "SNPs/datapoints\n")
+  nr <- nrow(object$data)
+  if (is.null(nr)) nr <- 0
+  cat(nr, "SNPs/datapoints\n")
   cat(nrow(object$TX), "gene transcripts\n")
-  tb <- sort(c(table(object$TX$gene_biotype)), decreasing = TRUE)
-  cat(paste(tb, names(tb), collapse = ", "), "\n")
+  if (nrow(object$TX) > 0) {
+    tb <- sort(c(table(object$TX$gene_biotype)), decreasing = TRUE)
+    cat(paste(tb, names(tb), collapse = ", "), "\n")
+  }
 }
 
